package tools.descartes.dql.connector.kieker.filter;


import kieker.analysis.IProjectContext;
import kieker.analysis.plugin.annotation.InputPort;
import kieker.analysis.plugin.annotation.OutputPort;
import kieker.analysis.plugin.annotation.Plugin;
import kieker.analysis.plugin.annotation.Property;
import kieker.analysis.plugin.filter.AbstractFilterPlugin;
import kieker.common.configuration.Configuration;
import kieker.common.record.IMonitoringRecord;
import kieker.common.record.controlflow.OperationExecutionRecord;


@Plugin(
		name = "Execution time filter",
		description = "Extracts execution time information from incoming monitoring records",
		outputPorts = {
				@OutputPort(name = ExecutionTimeFilter.OUTPUT_PORT_NAME_VALID,
						description = "Outputs OperationExecutionRecord exceeding the threshold",
						eventTypes = { IMonitoringRecord.class }),
				@OutputPort(name = ExecutionTimeFilter.OUTPUT_PORT_NAME_INVALID,
						description = "Outputs OperationExecutionRecord not exceeding the threshold",
						eventTypes = { IMonitoringRecord.class }),
				@OutputPort(name = ExecutionTimeFilter.OUTPUT_PORT_NAME_OTHER,
						description = "Outputs untouched IMonitoringRecord",
						eventTypes = { IMonitoringRecord.class }),
		},
		configuration = {
				@Property(name = ExecutionTimeFilter.CONFIG_PROPERTY_NAME_TS_NANOS, defaultValue = "0")
		}
		)

public class ExecutionTimeFilter extends AbstractFilterPlugin {

	public static final String INPUT_PORT_NAME = "newMonitoringRecord";
	public static final String OUTPUT_PORT_NAME_VALID = "validExeTimeRecords";
	public static final String OUTPUT_PORT_NAME_INVALID = "invalidExeTimeRecords";
	public static final String OUTPUT_PORT_NAME_OTHER = "forwardingNonOpExRecords";
	public static final String CONFIG_PROPERTY_NAME_TS_NANOS = "thresholdNanos";
	private final long rtThresholdNanos; 

	public ExecutionTimeFilter(final Configuration configuration, final IProjectContext projectContext) {
		super(configuration, projectContext);
		this.rtThresholdNanos = configuration.getLongProperty(CONFIG_PROPERTY_NAME_TS_NANOS);
	}


	@InputPort(
			name = ExecutionTimeFilter.INPUT_PORT_NAME,
			description = "Extract class-information from monitoring record",
			eventTypes = { IMonitoringRecord.class })
	public void runFilter(final Object record) {
		if (record instanceof OperationExecutionRecord) {
			final OperationExecutionRecord opExRecord = (OperationExecutionRecord) record;
			final long exTime = opExRecord.getTout()-opExRecord.getTin();			
			if (exTime > this.rtThresholdNanos) {
				super.deliver(OUTPUT_PORT_NAME_VALID, opExRecord);
			} else {
				//System.out.println("execution Time: " + exTime);
				super.deliver(OUTPUT_PORT_NAME_INVALID, opExRecord);
			}
		} else {
			super.deliver(OUTPUT_PORT_NAME_OTHER, record);
		}
	}


	@Override
	public Configuration getCurrentConfiguration() {
		final Configuration configuration = new Configuration();
		configuration.setProperty(CONFIG_PROPERTY_NAME_TS_NANOS,
				Long.toString(this.rtThresholdNanos));
		return configuration;
	}


}
